<?php
/*
Plugin Name: XMPP-Auth
Plugin URI: http://jehan.zemarmot.net
Description: Authentication to the website through Instant Messaging Authorization (XMPP XEP-0070).
Version: 0.2
Author: Jehan Hysseo
Author URI: http://jehan.zemarmot.net
License: AGPLv3
*/

/*
    Copyright (C) 2011 Jehan Pagès (IM: xmpp:hysseo@zemarmot.net)
    [ No email contact, but you can send message through Instant Messaging,
    and you are not force to add my presence for this.
    You can also just send a normal message. I try to answer every message. ]

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program (file LICENSE at the root of the source archive). 
    If not, see <http://www.gnu.org/licenses/>.
*/

//require_once(dirname(__FILE__) . '/plugged.php');

// If first run.
$certs_hashed = glob(dirname(__FILE__) . '/certs/*.0');
if (empty($certs_hashed))
{
	unset($certs_hashed);
	$pems = glob(dirname(__FILE__) . '/certs/*.pem');
	// TODO: readdir or opendir would be more memory efficient.
	foreach($pems as $cert)
	{
		$hash = shell_exec('openssl x509 -hash -noout -in "' . $cert . '"');
		if (!empty($hash))
			@symlink($cert, dirname(__FILE__) . '/certs/' . rtrim($hash) . '.0');
	}
}

function imauth_login_init()
{
	$configuration = get_option('imauth_configuration');
    if (isset($configuration['disable_login']) && $configuration['disable_login'])
        return;

	wp_enqueue_script('jquery');
	wp_enqueue_script('xmpp-auth',
	 // WP_PLUGIN_URL . '/someplugin/js/newscript.js', // old way, not SSL compatible
	  plugins_url('/xmpp-auth.js', __FILE__));
	wp_localize_script( 'xmpp-auth', 'objectL10n', array(
				'auth_id' => __('Authentication Identifier'),
				'conf_hide' => __('[Hide]'),
				'conf_show' => __('[Display]'),
				) );
}
add_action('login_init', 'imauth_login_init');

function imauth_admin_init()
{
    wp_enqueue_style('xmpp-auth', plugins_url('/xmpp-auth.css', __FILE__));
	wp_enqueue_script('jquery');
	wp_enqueue_script('xmpp-auth',
	 // WP_PLUGIN_URL . '/someplugin/js/newscript.js', // old way, not SSL compatible
	  plugins_url('/xmpp-auth.js', __FILE__));
	wp_localize_script( 'xmpp-auth', 'objectL10n', array(
				'auth_id' => __('Authentication Identifier'),
				'conf_hide' => __('[Hide]'),
				'conf_show' => __('[Display]'),
				) );
}
// TODO: actually I'd like to limit this one to the plugin configuration page.
// XXX: there are a wp_print_scripts/styles action, but they would load the scripts/styles all the time.
// I'll try and avoid this.
add_action('admin_init', 'imauth_admin_init');

function imauth_login_checkbox()
{
    $configuration = get_option('imauth_configuration');
    if (isset($configuration['disable_login']) && $configuration['disable_login'])
        return;

    $imauth = ! empty($_POST['imauth']);

	echo '<p id="imauth_transaction_id_p" ';
	if (!$imauth)
		echo 'style="display:none;"';
	echo '><label for="transaction_id" id="imauth_transaction_id_label">'
		. __('Authentication Identifier (Instant Messaging log-in only)') . '</label>'
		. '<br /><input type="password" name="transaction_id" id="transaction_id" class="input" value="" size="20" tabindex="20" /></p>';

	echo '<p class="forgetmenot"><label><input name="imauth" type="checkbox" id="imauth" value="forever" tabindex="90"'
		. ($imauth? " checked='checked'":'') . ' onclick="updateLoginPage();" /> '
		. esc_attr('Via Instant Messaging') . '&nbsp;</label></p>';

	echo '<script type="text/javascript">
		updateLoginPage();
		</script>';
}

add_action('login_form', 'imauth_login_checkbox');
// TODO: add checkbox/list to deactivate jabber auth, normal auth, or none (default none) in profile.

// "Reroute" a login process to IM authentication when the checkbox is checked.


function imauth_login_route($user, $username)
{
	$configuration = get_option('imauth_configuration');
    if (isset($configuration['disable_login']) && $configuration['disable_login'])
	{
		add_filter('authenticate', 'wp_authenticate_username_password', 20, 3);
		return;
	}

	if (!isset($error))
		$error = new WP_Error();
	if (!isset($_POST['imauth']) || empty($_POST['imauth']))
	{
		add_filter('authenticate', 'wp_authenticate_username_password', 20, 3);
		return;
	}
	if (empty($_POST['transaction_id']))
	{
		// Section 4.3 of XEP-0070: "transaction identifier SHOULD be provided by the human user who controls the HTTP Client."
		// I could though generate one from here instead, but I prefer this to be controlled by the user,
		// unless the web browser, hopefully in the future, includes IM control.
		$error->add('empty_transaction', __('Provide a Transaction ID of your choice.'));
		return $error;
	}
	if (empty($username))
	{
		$error->add('empty_username', __('<strong>ERROR</strong>: The username field is empty.'));
		return $error;
	}

	$userinfo = get_userdatabylogin($username);
	//$userdata = get_user_by('login', $username);

    require_once(dirname(__FILE__) . '/xmpp_utils.php');
	if (!$userinfo || empty($userinfo->jabber)
		|| !imauth_valid_jid($userinfo->jabber))
	{
		// On purpose I make 2 errors in one, not to leak too easily existence of a user or of a JID.
		$error->add('invalid_jid', __('<strong>ERROR</strong>: invalid user. Did you set a valid IM address in your profile?'));
		return $error;
	}

	// I have my own time limit system, so I deactivate the PHP one.
	// It avoids a ugly PHP error on the page and is handed nicely by Wordpress error system.
	set_time_limit(0);
    $xs = NULL;
    if (!empty($configuration['component']))
    {
        require_once(dirname(__FILE__) . '/xmpp_component.php');
        $xs = new xmpp_component($configuration['component'], $configuration['component_secret'],
                $configuration['component_server'], $configuration['component_port']);
    }
    else
    {
        require_once(dirname(__FILE__) . '/xmpp_stream.php');
        $configuration = get_option('imauth_configuration');
        $xs = new xmpp_stream ($configuration['node'],
                $configuration['domain'], $configuration['password'],
                'bot', $configuration['server'], $configuration['port']);
    }

	if (!$xs->log ())
	{
		$error->add('xmpp_connect', __('<strong>ERROR</strong>: connection issue occurred.'));
		return $error;
	}

	if ($xs->http_auth(get_site_url(), 'POST', $userinfo->jabber, $_POST['transaction_id'], 
		sprintf(__("Someone (maybe you) has requested access to \"%s\" at address:\n\n\t%s\n"), get_bloginfo('name'), get_site_url()))
		&& $xs->quit ())
	{
		//status_header(202);
		$user =  new WP_User($userinfo->ID);
		//if (! empty($_POST['rememberme']))
		//	$remember = $_POST['rememberme'] ? true : false;
		// else
		//	$remember = false;
		//wp_set_auth_cookie($user->ID, $remember, $secure_cookie); // secure cookie?
		return $user;
	}
	else
	{
		$error->add('xmpp_connect', __('<strong>ERROR</strong>: IM Request Failure.'));
		return $error;
	}
}

// I want my rerouting to be made at pretty high priority and only 1 parameter is needed (password does not matter).
//add_filter('authenticate', 'imauth_login_route', 0, 1);
//add_action('wp_authenticate', 'imauth_login_route', 0, 1);
//add_action('login_init', 'imauth_login_route', 0, 3);

// I remove the "normal authentication" hook and will call it myself from my own hook.
remove_filter('authenticate', 'wp_authenticate_username_password', 20, 3);
//remove_all_filters('authenticate');
add_filter('authenticate', 'imauth_login_route', 0, 2);

/**********************\
// Configuration Page \\
\**********************/

function imauth_configuration_page () // {{{
{
	global $wpdb;
	add_option('imauth_configuration', array (), 'deprecated', 'yes');
	add_option ('imauth_transactions', array (), 'deprecated', 'yes'); // Running transactions not yet aknowledged.

	if (isset($_POST['update_configuration']))
	{
		require_once(dirname(__FILE__) . '/xmpp_stream.php');
		$configuration['node'] = strip_tags (trim($_POST['node']));
		$configuration['domain'] = strip_tags (trim ($_POST['domain']));
		$configuration['password'] = strip_tags (trim($_POST['password']));
        $configuration['component'] = strip_tags(trim($_POST['component']));
        $configuration['component_secret'] = strip_tags(trim($_POST['component_secret']));
        $configuration['component_server'] = strip_tags(trim($_POST['component_server']));
        $component_port = strip_tags(trim($_POST['component_port']));
		if (is_numeric ($component_port))
            $configuration['component_port'] = intval($component_port);
		else
			$configuration['component_port'] = '';

		$configuration['disable_login'] = isset($_POST['disable_login']);
		$configuration['disable_comment'] = isset($_POST['disable_comment']);

		$posted_server = strip_tags (trim($_POST['server']));

		$configuration['server'] = $posted_server;

		$posted_port = strip_tags (trim($_POST['port']));
		if (is_numeric ($posted_port))
			$configuration['port'] = intval ($posted_port);
		else
			$configuration['port'] = '';

        $xs = NULL;
        if (!empty($configuration['component']))
        {
            require_once(dirname(__FILE__) . '/xmpp_component.php');
            $xs = new xmpp_component($configuration['component'], $configuration['component_secret'],
                $configuration['component_server'], $configuration['component_port']);
        }
        else
            $xs = new xmpp_stream ($configuration['node'],
                    $configuration['domain'], $configuration['password'],
                    'bot', $configuration['server'], $configuration['port']);
		
		if ($xs->log ()
			&& $xs->quit ())
		{
			update_option('imauth_configuration', $configuration);
			echo '<div class="updated"><p>' . __('Configuration tested and saved') . '</p></div>';
		}
		else
			echo '<div class="updated"><p>' . __('Configuration not saved. The following error occured:<br />') . $xs->last_error . '</p></div>';
			
	}
	else
		// If we are just displaying the page or if we reset to last saved config, we first load up the options array.
		$configuration = get_option('imauth_configuration');

    $as_component = isset($configuration['component']) && !empty($configuration['component']);
	//now we drop into html to display the option page form.
	?>
	<div class="wrap">
		<h2><?php echo _e('IM configuration') ?></h2>
		<form method="post" action="">
			<fieldset
                <?php echo $as_component? "style='display:none'" : ''; ?>
                id='xmppauth-bot-conf' class="imauth-options">
				<legend onclick="switchComponentConfiguration('<?php echo $configuration['component']; ?>');"><?php _e('Bot Account'); ?> <a id="xmppauth_component"><?php _e('[Switch to Component]'); ?></a></legend>
				<p><label>
				<?php _e('Login (bare jid)') ?><br />
					<input name="node"
						type="text"
						id="node"
						value="<?php echo $configuration['node']; ?>"
						size="29" /></label>
				
				<label>@
					<input name="domain"
						type="text"
						id="domain"
						value="<?php echo $configuration['domain']; ?>"
						size="28" />
				</label></p>

				<p><label>
				<?php _e('Password') ?><br />
					<input name="password"
						type="password"
						id="password"
						value="<?php echo $configuration['password']; ?>"
						size="60" />
				</label></p>

                <fieldset class="imauth-advanced-options">
                    <legend onclick="switchAdvancedConfiguration();"><?php _e('Advanced Connection Parameters');?> <a id="xmppauth_display"><?php _e('[Display]'); ?></a></legend>
                    <div id="xmppauth_advanced_conf" style="display:none">
                            <p><em><?php _e("These are advanced settings. If you don't understand them, they are probably useless and default values will be enough.") ?> </em></p>
                        <?php 
                        if (class_exists ("NET_DNS_Resolver") || function_exists ("dns_get_record"))
                        {
                            ?>
                        <p><em><?php _e('Note that SRV Records will be used by default.') ?></em></p>
                        <?php
                        }
                        else
                        {
                            ?>
                            <p><em><?php _e('SRV Records discovery option is not enabled because the PEAR module NET_DNS is not installed on this server.') ?></em></p>
                            <?php
                        }
                ?>

                        <p><label>
                        <?php _e('Server') ?><br />
                            <input name="server"
                                type="text"
                                id="server"
                                value="<?php echo $configuration['server']; ?>"
                                size="60" />
                        </label></p>

                        <p><label>
                        <?php _e('Port') ?><br />
                            <input name="port"
                                type="text"
                                id="port"
                                value="<?php echo $configuration['port']; ?>"
                                size="60" />
                        </label></p>
                    </div>
                </fieldset>
			</fieldset>
			<fieldset
                <?php echo $as_component? '' : "style='display:none'"; ?>
                id='xmppauth-component-conf' class="imauth-options">
				<legend onclick="switchComponentConfiguration('<?php echo $configuration['component']; ?>');"><?php _e('Component Account'); ?> <a id="xmppauth_component"><?php _e('[Switch to Bot]'); ?></a></legend>
                <p><em><?php _e("Register as a component instead of using a bot. This requires you to run your own server and configure it. If you don't understand what is a component, this feature is probably not for you and you should come back on bot mode.") ?> </em></p>
                <p><strong><?php _e("NOTE: this is an experimental support.") ?> </strong></p>
				<p><label>
				<?php _e('Component bare jid') ?><br />
					<input name="component"
						type="text"
						id="component"
						value="<?php echo $configuration['component']; ?>"
						size="29" /></label></p>
				
				<p><label>
				<?php _e('Secret') ?><br />
					<input name="component_secret"
						type="password"
						id="component_secret"
						value="<?php echo $configuration['component_secret']; ?>"
						size="60" />
				</label></p>

				<p><label>
				<?php _e('Server') ?>
					<input name="component_server"
						type="text"
						id="component_server"
						value="<?php echo $configuration['component_server']; ?>"
						size="29" /></label>

				<label><?php _e('Port') ?>
					<input name="component_port"
						type="text"
						id="component_port"
						value="<?php echo $configuration['component_port']; ?>"
						size="29" /></label></p>
            </fieldset>

			<fieldset class="imauth-options">
				<legend><?php _e('Features');?></legend>
                <p><label><input name="disable_login" type="checkbox" id="disable_login" value="disable_login"
                <?php echo isset($configuration['disable_login']) && $configuration['disable_login']? 'checked="checked"' : ''; ?>
                /> <?php _e('Disable Login via XMPP'); ?></label></p>
                <p><label><input name="disable_comment" type="checkbox" id="disable_comment" value="disable_comment"
                <?php echo isset($configuration['disable_comment']) && $configuration['disable_comment']? 'checked="checked"' : ''; ?>
                /> <?php _e('Disable Commenters Authentication via XMPP'); ?></label></p>
			</fieldset>

			<?php echo apply_filters('imauth_additional_configuration', ''); ?>

			<div class="submit">
				<input type="submit"
					name="update_configuration"
					value="<?php _e('Update') ?>"
					style="font-weight:bold;" />
				<input type="submit"
					name="reset_configuration"
					value="<?php _e('Reset') ?>"
					style="font-weight:bold;font-style:italic" />
			</div>
		</form>    		
	</div>
	<?php	
} // }}}

function imauth_menu () // {{{
{
	if (function_exists('current_user_can'))
	{
		if (!current_user_can('manage_options'))
			return;
	}
	else
	{
		global $user_level;
		get_currentuserinfo ();
		if ($user_level < 8)
			return;
	}
	if (function_exists ('add_submenu_page'))
		add_submenu_page('plugins.php', __('IM authentication'), __('XMPP Authentication'), 'edit_plugins', 'xmpp-auth-conf', 'imauth_configuration_page');
} // }}}

// Install the configuration page.
add_action ('admin_menu', 'imauth_menu');


/***************************** COMMENTING ***********************/

function xmppauth_comment_form_fields($fields)
{
	$configuration = get_option('imauth_configuration');
    $jid_validate = !isset($configuration['disable_comment']) || !$configuration['disable_comment'];

	$comment_author_jid = isset($_COOKIE['comment_author_jid_' . COOKIEHASH]) ? $_COOKIE['comment_author_jid_' . COOKIEHASH] : '';
		
	// XXX: I use the email class as it won't display well for some theme (example: twentyeleven) if there is no.
	// XXX: the aria-required parameter is apparently for screen readers. Good for accessibility.
	// TODO: get a default value in cookie if the non subscriber user previously commented OR if this is a logged-in user?
	$fields['jid'] = '<p class="comment-form-email"><label for="jid" title="Jabber ID (will not be published)">' . __( 'JID' ) . '</label>'
        . ($jid_validate? '<span class="required">*</span>' : '')
        . '<input id="jid" name="jid" type="text" value="' . $comment_author_jid . '" size="30" aria-required="true" /></p>';
	return $fields;
}

add_filter('comment_form_default_fields', 'xmppauth_comment_form_fields', 10, 1);

function xmppauth_validate_comment($approved, $commentdata)
{
	$configuration = get_option('imauth_configuration');

    // If commenter authentication is deactivated, I don't do a thing.
    if (isset($configuration['disable_comment']) && $configuration['disable_comment'])
        return $approved;

	// I don't change a comment previously disapproved by another system
	// nor a comment from a registered user.
	if ($approved === 'spam' || $approved == 'trash'
		|| is_user_logged_in() )
		return $approved;

	$comment_content = $commentdata['comment_content'];
	// For now I will simply disapprove a comment.
	// I wonder if I should not directly trash it (return "trash" instead of false).
    require_once(dirname(__FILE__) . '/xmpp_utils.php');
	if (!imauth_valid_jid($_POST['jid']))
		wp_die(__('Error: please enter a valid JID. Comment:<br />'). esc_attr($comment_content));

	set_time_limit(0); // XXX: shouldn't this be in my_socket?
    $xs = NULL;
    if (!empty($configuration['component']))
    {
        require_once(dirname(__FILE__) . '/xmpp_component.php');
        $xs = new xmpp_component($configuration['component'], $configuration['component_secret'],
                $configuration['component_server'], $configuration['component_port']);
    }
    else
    {
        require_once(dirname(__FILE__) . '/xmpp_stream.php');
        $xs = new xmpp_stream ($configuration['node'],
                $configuration['domain'], $configuration['password'],
                'bot', $configuration['server'], $configuration['port']);
    }

	if (!$xs->log ())
	{
		// XXX: it does not mean that the user's JID is wrong.
		//I don't save the comment when it happens.
		wp_die(__('Error: internal system error. Please retry later. Comment:<br />') . esc_attr($comment_content));
	}

	// I use the comment content as transaction id.
	if ($xs->http_auth(get_permalink($commentdata['comment_post_ID']), 'POST', $_POST['jid'], $comment_content, 
		sprintf(__("Someone (maybe you) is commenting on the website \"%s\" (%s)"), get_bloginfo('name'), get_site_url()))
		&& $xs->quit ())
	{
		return $approved;
	}
	else
	{
		// return "trash";
		wp_die(__('Error: the comment was not validated. Are you sure your JID is valid? Comment:<br />') . esc_attr($comment_content));
	}
}

// I want to be last on the list of approval.
add_filter('pre_comment_approved', 'xmppauth_validate_comment', 100, 2);

function xmppauth_save_comment_jid($comment_id)
{
	$jid = stripslashes(esc_attr($_POST['jid']));

	// See update_comment_meta.
	if (isset($_POST['jid']))
		add_comment_meta($comment_id, 'jid', $jid, true);

	$comment_cookie_lifetime = apply_filters('comment_cookie_lifetime', 30000000);
	setcookie('comment_author_jid_' . COOKIEHASH, $jid, time() + $comment_cookie_lifetime, COOKIEPATH, COOKIE_DOMAIN);
}

add_action('comment_post', 'xmppauth_save_comment_jid', 10, 1);

/******************* Plugins Page ************/

function xmppauth_plugin_action_links($actions, $plugin_file )
{
	if ($plugin_file == plugin_basename(dirname(__FILE__) .'/xmpp-auth.php' ))
    {
		$actions['settings'] = '<a href="plugins.php?page=xmpp-auth-conf">'.__('Settings').'</a>';
	}

	return $actions;
}

add_filter('plugin_action_links', 'xmppauth_plugin_action_links', 10, 2 );


/****** Admin notice *****/
$not_configured = !get_option('imauth_configuration');
function xmppauth_not_configured_notice()
{
    echo "<div id='xmppauth-warning' class='updated fade'><p><strong>".__('XMPP Authentication is not configured yet!')."</strong> ".sprintf(__('You must <a href="%1$s">configure it</a> for it to work.'), "plugins.php?page=xmpp-auth-conf")."</p></div>";
}
if ($not_configured)
		add_action('admin_notices', 'xmppauth_not_configured_notice');


function rename_jabber_label($current)
{
    return 'Standard IM (Jabber / Google Talk / LiveJournal…)';
}

add_filter('user_jabber_label', 'rename_jabber_label', 30, 1);
?>
